
/*
 *  PRUEBA ESCALON - © Mecatronium Chips 2020 - Victoriano Montemayor 
 *  victoriano@mecatronium.com
 *  v1.1 - Jul 2020 - Correccion variable int a long en funcion readValue. (Gracias @Jose Mariano Uvalle)
 *  
 *  Este programa genera una prueba escalon configurable por medio de variables 
 *  Para el kit de Control de Iluminación con Fotoresistencia y Matriz de LED 8x8
 *  
 *  CONEXIONES
 *  
 *  pin A0 se conecta a Fotoresistencia
 *  pin A1 se conecta a Potenciometro
 *  
 *  pin 10 se conecta a CS
 *  pin 11 se conecta a CLK 
 *  pin 12 se conecta a DIN
 * 
 */


#include "LedControl.h"
#include <PID_v1.h>

#define sensorPin A0    //Pin al que se conecta la Fotoresistencia
#define potPin A1       //Pin al que se conecta el Potenciómetro

#define NUMVALORES 100  //Numero de valores a tomar para promediar
#define MS_DELAY 500    //Tiempo en ms a esperar antes de aplicar el escalón (tiempo para estabilizar medicion de sensor)
#define MS_HOLD 2000    //Tiempo en ms a mantener escalón
#define STEP_VALUE 100  //Valor de salida de prueba escalón (0% - 100%) en este caso equivalen de 0 a 64 LEDs

int sensorValue = 0;    //Variable para guardar la lectura de la Fotoresistencia
int values[NUMVALORES]; //Arreglo con los ultimos NUMVALORES valores de la Fotoresistencia

LedControl lc = LedControl(12,11,10,1); //Inicializar Control de Matriz de LED

//Inicializar arreglo en 0's
void initValues(){
  for(int i = 0; i < NUMVALORES; i++){
    values[i] = 0;
  }
}

void setup() {

  pinMode(A0, INPUT); //Configurar como entrada
  Serial.begin(9600); //Inicializar puerto serial
  
  lc.shutdown(0,false); //Despertar Matriz LED
  lc.setIntensity(0,4); //Establecer intensidad de LED
  lc.clearDisplay(0);   //Apagar todos los LED

  initValues();
}

int numLeds = 0;  //Número de LEDs a encender
int counter = 0;  //Contador

void loop() {

  if(counter < MS_DELAY) {
    numLeds = 0; 
  } else if(counter < MS_DELAY + MS_HOLD) {
    numLeds = calcularLEDS(STEP_VALUE);
  } else {
    numLeds = 0;
  }
  
  prendeLeds(numLeds);
  counter++;

  //Graficar
  Serial.print(numLeds);
  Serial.print(",");
  Serial.println(readValue());
  
  delay(1);
}

//Leer el valor actual del sensor y promediar con los ultimos NUMVALORES valores
//para crear un valor mas estable
int readValue(){
  //Recorrer valores
  for(int i = 0; i < NUMVALORES - 1; i++){
    values[i] = values[i+1];
  }

  //Leer el mas nuevo
  values[NUMVALORES - 1] = analogRead(sensorPin);

  //Calcular y regresar promedio
  long suma = 0;
  for(int i = 0; i < NUMVALORES; i++){
    suma += values[i];
  }
  int promedio = suma / NUMVALORES;
  
  return promedio;
}

//Prender el numeroDeLeds indicado
void prendeLeds(int numeroDeLeds){
  for(int i = 0; i < 8; i++){
    for (int j = 0; j < 8; j++){
      if( (i*8) + j <= numeroDeLeds){
        lc.setLed(0, i, j, true); 
      } else {
        lc.setLed(0, i, j, false);
      }
    }
  }
}

//Convertir 0% a 100%  >>  0 a 64 LED
int calcularLEDS(int valorEnPorcentaje){
  return (valorEnPorcentaje * 64) / 100;
}
